# -*- coding: utf-8 -*-
from abjad import *
Component = scoretools.Component
Selection = selectiontools.Selection
selector = select().by_leaf(flatten=True)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_01():
    r'''Unincorporated leaves do not share a logical voice.
    Unicorporated leaves do not share a root component.
    False if not allow orphans; True if allow orphans.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    assert Selection._all_are_components_in_same_logical_voice(notes)
    assert not Selection._all_are_components_in_same_logical_voice(
        notes, allow_orphans=False)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_02():
    r'''Container and leaves all logical voice.
    '''

    container = Container("c'8 d'8 e'8 f'8")

    r'''
    {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert Selection._all_are_components_in_same_logical_voice(
        select(container).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_03():
    r'''Tuplet and leaves all logical voice.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")

    r'''
    \times 2/3 {
        c'8
        d'8
        e'8
    }
    '''

    assert Selection._all_are_components_in_same_logical_voice(
        select(tuplet).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_04():
    r'''Voice and leaves all appear in same logical voice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")

    r'''
    \new Voice {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert Selection._all_are_components_in_same_logical_voice(
        select(voice).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_05():
    r'''Anonymous staff and leaves all appear in same logical voice.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")

    r'''
    \new Staff {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert Selection._all_are_components_in_same_logical_voice(
        select(staff).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_06():
    r'''Voice, sequential and leaves all appear in same logical voice.
    '''

    voice = Voice(r'''
        {
            c'8
            d'8
            e'8
            f'8
        }
        {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    assert Selection._all_are_components_in_same_logical_voice(
        select(voice).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_07():
    r'''Anonymous voice, tuplets and leaves all appear in same logical voice.
    '''

    voice = Voice(r'''
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        \times 2/3 {
            f'8
            g'8
            a'8
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8
            }
        }
        '''
        )

    assert Selection._all_are_components_in_same_logical_voice(
        select(voice).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_08():
    r'''Logical voice does not extend across anonymous voices.
    '''

    staff = Staff(r'''
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }
        \new Voice {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(staff)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)
    assert not Selection._all_are_components_in_same_logical_voice(staff[:])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_09():
    r'''Logical voice encompasses across like-named voices.
    '''

    staff = Staff(r'''
        \context Voice = "foo" {
            c'8
            d'8
            e'8
            f'8
        }
        \context Voice = "foo" {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8
                d'8
                e'8
                f'8
            }
            \context Voice = "foo" {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(staff)
    assert Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_10():
    r'''Logical voice does not extend across differently named voices.
    '''

    staff = Staff(r'''
        \context Voice = "foo" {
            c'8
            d'8
        }
        \context Voice = "bar" {
            e'8
            f'8
        }
        ''')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "foo" {
                c'8
                d'8
            }
            \context Voice = "bar" {
                e'8
                f'8
            }
        }
        '''
        )

    leaves = selector(staff)
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_11():
    r'''Logical voice does not across anonymous voices.
    Logical voice does not extend across anonymous staves.
    '''

    container = Container(r'''
        \new Staff {
            \new Voice {
                c'8
                d'8
            }
        }
        \new Staff {
            \new Voice {
                e'8
                f'8
            }
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff {
                \new Voice {
                    c'8
                    d'8
                }
            }
            \new Staff {
                \new Voice {
                    e'8
                    f'8
                }
            }
        }
        '''
        )

    leaves = selector(container)
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_12():
    r'''Logical voice does not extend across anonymous voices.
    Logical voice does not extend across anonymous staves.
    '''

    container = Container(r'''
        \new Staff <<
            \new Voice {
                c'8
                d'8
            }
            \new Voice {
                e'8
                f'8
            }
        >>
        \new Staff <<
            \new Voice {
                g'8
                a'8
            }
            \new Voice {
                b'8
                c''8
            }
        >>
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff <<
                \new Voice {
                    c'8
                    d'8
                }
                \new Voice {
                    e'8
                    f'8
                }
            >>
            \new Staff <<
                \new Voice {
                    g'8
                    a'8
                }
                \new Voice {
                    b'8
                    c''8
                }
            >>
        }
        '''
        )

    leaves = selector(container)
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:4])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_13():
    r'''Anonymous voice, sequentials and leaves all appear in same
    logical voice.
    '''

    voice = Voice(r'''
        {
            c'8
            d'8
        }
        {
            e'8
            f'8
        }
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }
        '''
        )

    leaves = selector(voice)
    assert Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_14():
    r'''Logical voice can extend across like-named staves.
    Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        \context Staff = "foo" {
            c'8
            cs'8
            d'8
            ef'8
        }
        \context Staff = "foo" {
            e'8
            f'8
            fs'8
            g'8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "foo" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Staff = "foo" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_15():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        {
            c'8
            d'8
            e'8
            f'8
        }
        \new Voice {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_16():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }
        {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_17():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        {
            c'8
            d'8
            e'8
            f'8
        }
        \context Voice = "foo" {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    r'''
    {
        {
            c'8
            d'8
            e'8
            f'8
        }
        \context Voice = "foo" {
            g'8
            a'8
            b'8
            c''8
        }
    }
    '''

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_18():
    r'''Logical voice can not extend acrossdifferently named implicit voices.
    '''

    container = Container(r'''
        \context Voice = "foo" {
            c'8
            d'8
            e'8
            f'8
        }
        {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Voice = "foo" {
                c'8
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_19():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        {
            c'8
            d'8
            e'8
            f'8
        }
        \new Staff {
            g'8
            a'8
            b'8
            c''8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_20():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff {
                c'8
                cs'8
                d'8
                ef'8
            }
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_21():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        d'8
        ef'8
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_22():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        \new Voice {
            c'8
            cs'8
            d'8
            ef'8
        }
        e'8
        f'8
        fs'8
        g'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_23():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        d'8
        ef'8
        \context Voice = "foo" {
            e'8
            f'8
            fs'8
            g'8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
            \context Voice = "foo" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_24():
    r'''Logical voice can not extend across differently named implicit voices.
    NOTE: THIS IS THE LILYPOND LACUNA.
    LilyPond *does* extend logical voice in this case.
    Abjad does not.
    '''

    container = Container(r'''
        \context Voice = "foo" {
            c'8
            cs'8
            d'8
            ef'8
        }
        e'8
        f'8
        fs'8
        g'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Voice = "foo" {
                c'8
                cs'8
                d'8
                ef'8
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_25():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        d'8
        ef'8
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_26():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        g'8
        a'8
        b'8
        c''8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_27():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    voice = Voice([Note(n, (1, 8)) for n in range(4)])
    container = Container([voice])
    notes = [Note(n, (1, 8)) for n in range(4, 8)]
    container = Container([container] + notes)

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                \new Voice {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_28():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    voice = Voice([Note(n, (1, 8)) for n in range(4)])
    voice.name = 'foo'
    q = Container([voice])
    notes = [Note(n, (1, 8)) for n in range(4, 8)]
    container = Container([q] + notes)

    r'''
    {
        {
            \context Voice = "foo" {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        e'8
        f'8
        fs'8
        g'8
    }
    '''

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_29():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    voice_1 = Voice([Note(n, (1, 8)) for n in range(4)])
    voice_1.name = 'foo'
    voice_2 = Voice([voice_1])
    voice_2.name = 'bar'
    notes = [Note(n, (1, 8)) for n in range(4, 8)]
    container = Container([voice_2] + notes)

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Voice = "bar" {
                \context Voice = "foo" {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_30():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    voice_1 = Voice([Note(n, (1, 8)) for n in range(4)])
    voice_2 = Voice([voice_1])
    notes = [Note(n, (1, 8)) for n in range(4, 8)]
    container = Container([voice_2] + notes)

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Voice {
                \new Voice {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_31():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    notes = [Note(n, (1, 8)) for n in range(4)]
    voice_1 = Voice(Note(12, (1, 8)) * 4)
    voice_2 = Voice(Note(0, (1, 8)) * 4)
    container = Container([voice_1, voice_2])
    container.is_simultaneous = True
    container = Container(notes + [container])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
            <<
                \new Voice {
                    c''8
                    c''8
                    c''8
                    c''8
                }
                \new Voice {
                    c'8
                    c'8
                    c'8
                    c'8
                }
            >>
        }
        '''
        )

    leaves = selector(container)
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:8])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[4:])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_32():
    r'''Logical voice can not extend across differently named implicit voices.
    '''

    container = Container(r'''
        <<
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        >>
        af'8
        a'8
        bf'8
        b'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            <<
                \new Voice {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
                \new Voice {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            >>
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )

    leaves = selector(container)
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:8])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[4:])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_33():
    r'''Logical voice does extend across gaps.
    Logical voice can not extend across differently named voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        \new Voice {
            d'8
            ef'8
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
            af'8
            a'8
        }
        bf'8
        b'8
        ''')

    outer = (0, 1, 10, 11)
    middle = (2, 3, 8, 9)
    inner = (4, 5, 6, 7)

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            \new Voice {
                d'8
                ef'8
                \new Voice {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                af'8
                a'8
            }
            bf'8
            b'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in middle])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in inner])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:4])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_34():
    r'''Logical voice does extend across gaps.
    Logical voice can not extend across differently named implicit voices.
    '''

    staff = Staff(r'''
        c'8
        cs'8
        \new Staff {
            d'8
            ef'8
            \new Staff {
                e'8
                f'8
                fs'8
                g'8
            }
            af'8
            a'8
        }
        bf'8
        b'8
        ''')

    outer = (0, 1, 10, 11)
    middle = (2, 3, 8, 9)
    inner = (4, 5, 6, 7)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            \new Staff {
                d'8
                ef'8
                \new Staff {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                af'8
                a'8
            }
            bf'8
            b'8
        }
        '''
        )

    leaves = selector(staff)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in middle])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in inner])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:4])


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_35():
    r'''Containers and leaves all appear in same logical voice.
    '''

    container = Container(r'''
        c'8
        cs'8
        {
            d'8
            ef'8
            {
                e'8
                f'8
                fs'8
                g'8
            }
            af'8
            a'8
        }
        bf'8
        b'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            {
                d'8
                ef'8
                {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                af'8
                a'8
            }
            bf'8
            b'8
        }
        '''
        )

    assert Selection._all_are_components_in_same_logical_voice(
        select(container).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_36():
    r'''Tuplets and leaves all appear in same logical voice.
    '''

    a = scoretools.FixedDurationTuplet(
        Duration(3, 8), "e'8 f'8 fs'8 g'8")
    b = scoretools.FixedDurationTuplet(
        Duration(3, 8), "d'8 ef'8 af'8 a'8")
    t = scoretools.FixedDurationTuplet(
        Duration(3, 8), "c'8 cs'8 bf'8 b'8")
    b.insert(2, a)
    t.insert(2, b)
    b.target_duration = Duration(6, 8)
    t.target_duration = Duration(9, 8)

    assert format(t) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 9/10 {
            c'8
            cs'8
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7 {
                d'8
                ef'8
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                af'8
                a'8
            }
            bf'8
            b'8
        }
        '''
        )

    assert Selection._all_are_components_in_same_logical_voice(
        select(t).by_class())


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_37():
    r'''Logical voice can not extend across differently named voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        {
            {
                \context Voice = "foo" {
                    d'8
                    ef'8
                    e'8
                    f'8
                }
            }
        }
        fs'8
        g'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            {
                {
                    \context Voice = "foo" {
                        d'8
                        ef'8
                        e'8
                        f'8
                    }
                }
            }
            fs'8
            g'8
        }
        '''
        )

    outer = (0, 1, 6, 7)
    inner = (2, 3, 4, 5)

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in inner])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_38():
    r'''Logical voice does not extend over differently named voices.
    '''

    container = Container(r'''
        {
            {
                \context Voice = "foo" {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
        }
        e'8
        f'8
        fs'8
        g'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                {
                    \context Voice = "foo" {
                        c'8
                        cs'8
                        d'8
                        ef'8
                    }
                }
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_39():
    r'''Can not nest across differently named implicit voices.
    '''

    container = Voice(
        r'''
        {
            {
                {
                    c'8
                    cs'8
                    \new Voice {
                        d'8
                        ef'8
                        e'8
                        f'8
                    }
                    fs'8
                    g'8
                }
            }
        }
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    {
                        c'8
                        cs'8
                        \new Voice {
                            d'8
                            ef'8
                            e'8
                            f'8
                        }
                        fs'8
                        g'8
                    }
                }
            }
        }
        '''
        )

    outer = (0, 1, 6, 7)
    inner = (2, 3, 4, 5)

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in inner])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_40():
    r'''Logical voice can not extend across differently named voices.
    '''

    voice = Voice(r'''
        c'8
        cs'8
        {
            d'8
            ef'8
            {
                e'8
                f'8
                \context Voice = "bar" {
                    fs'8
                    g'8
                    af'8
                    a'8
                }
                bf'8
                b'8
            }
            c''8
            cs''8
        }
        d''8
        ef''8
        ''')
    voice.name = 'foo'

    assert format(voice) == stringtools.normalize(
        r'''
        \context Voice = "foo" {
            c'8
            cs'8
            {
                d'8
                ef'8
                {
                    e'8
                    f'8
                    \context Voice = "bar" {
                        fs'8
                        g'8
                        af'8
                        a'8
                    }
                    bf'8
                    b'8
                }
                c''8
                cs''8
            }
            d''8
            ef''8
        }
        '''
        )

    outer = (0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 15)
    inner = (6, 7, 8, 9)

    leaves = selector(voice)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in inner])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_41():
    r'''Logical voice can not extend across differently named anonymous voices.
    '''

    container = Container(r'''
        \new Voice {
            c'8
            cs'8
            d'8
            ef'8
        }
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
        af'8
        a'8
        bf'8
        b'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(leaves[:4])
    assert Selection._all_are_components_in_same_logical_voice(leaves[4:8])
    assert Selection._all_are_components_in_same_logical_voice(leaves[8:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:8])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[4:])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)


def test_selectiontools_Selection__all_are_components_in_same_logical_voice_42():
    r'''Logical voice can not extend across differently named anonymous voices.
    '''

    container = Container(r'''
        c'8
        cs'8
        <<
            \new Voice {
                d'8
                ef'8
                e'8
                f'8
            }
            \new Voice {
                fs'8
                g'8
                af'8
                a'8
            }
        >>
        bf'8
        b'8
        ''')

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            <<
                \new Voice {
                    d'8
                    ef'8
                    e'8
                    f'8
                }
                \new Voice {
                    fs'8
                    g'8
                    af'8
                    a'8
                }
            >>
            bf'8
            b'8
        }
        '''
        )

    outer = (0, 1, 10, 11)

    leaves = selector(container)
    assert Selection._all_are_components_in_same_logical_voice(
        [leaves[i] for i in outer])
    assert Selection._all_are_components_in_same_logical_voice(leaves[2:6])
    assert Selection._all_are_components_in_same_logical_voice(leaves[6:10])
    assert not Selection._all_are_components_in_same_logical_voice(leaves[:6])
    assert not Selection._all_are_components_in_same_logical_voice(leaves)



# -*- coding: utf-8 -*-
import random
from abjad import *


def test_selectortools_Selector_run_selectors_01():

    staff = Staff("c'4 d'8 e'8 f'4 g'8 a'4 b'8 c'8")

    selector = selectortools.Selector()
    logical_tie_selector = selector.by_logical_tie()
    pitched_selector = logical_tie_selector.by_pitch('C4')
    duration_selector = logical_tie_selector.by_duration('==', (1, 8))
    contiguity_selector = duration_selector.by_contiguity()

    selectors = [
        selector,
        logical_tie_selector,
        pitched_selector,
        duration_selector,
        contiguity_selector,
        ]

    all_results = []

    for _ in range(10):
        result = selectortools.Selector.run_selectors(staff, selectors)
        all_results.append(result)
        random.shuffle(selectors)

    for result in all_results:
        assert len(result) == 5
        assert selector in result
        assert logical_tie_selector in result
        assert pitched_selector in result
        assert duration_selector in result
        assert contiguity_selector in result

    pairs = sequencetools.Sequence(all_results).nwise()
    for results_one, results_two in pairs:
        for selector in selectors:
            assert results_one[selector] == results_two[selector]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_lilypondproxytools_LilyPondGrobNameManager___getattr___01():
    r'''Getting unknown grob name raises exception.
    '''

    note = Note("c'8")
    assert pytest.raises(Exception, 'override(note).foo')



# -*- coding: utf-8 -*-


def all_are_pairs_of_types(argument, first_type, second_type):
    r'''Is true when `argument` is an iterable collection whose members are all
    of length 2, and where the first member of each pair is an instance of
    `first_type` and where the second member of each pair is an instance of
    `second_type`. Otherwise false.

    ..  container:: example

        ::

            >>> items = [(1., 'a'), (2.1, 'b'), (3.45, 'c')]
            >>> mathtools.all_are_pairs_of_types(items, float, str)
            True

        ::

            >>> mathtools.all_are_pairs_of_types('foo', float, str)
            False

    ..  container:: example

        Is true when `argument` is empty:

        ::

            >>> mathtools.all_are_pairs_of_types([], float, str)
            True

    Returns true or false.
    '''
    try:
        return all(
            (len(_) == 2 and
            isinstance(_[0], first_type) and
            isinstance(_[1], second_type))
            for _ in argument
            )
    except (KeyError, TypeError):
        return False



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_KeySignature_name_01():

    assert KeySignature('e', 'major').name == 'E major'
    assert KeySignature('E', 'major').name == 'E major'

    assert KeySignature('e', 'minor').name == 'e minor'
    assert KeySignature('E', 'minor').name == 'e minor'



# -*- coding: utf-8 -*-
import collections
from abjad.tools import datastructuretools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Performer(AbjadObject):
    r'''A performer.

    ..  container:: example

        ::

            >>> performer = instrumenttools.Performer(name='flutist')
            >>> performer.instruments.append(instrumenttools.Flute())
            >>> performer.instruments.append(instrumenttools.Piccolo())

        ::

            >>> print(format(performer))
            instrumenttools.Performer(
                name='flutist',
                instruments=instrumenttools.InstrumentList(
                    [
                        instrumenttools.Flute(
                            instrument_name='flute',
                            short_instrument_name='fl.',
                            instrument_name_markup=markuptools.Markup(
                                contents=['Flute'],
                                ),
                            short_instrument_name_markup=markuptools.Markup(
                                contents=['Fl.'],
                                ),
                            allowable_clefs=instrumenttools.ClefList(
                                [
                                    indicatortools.Clef(
                                        name='treble',
                                        ),
                                    ]
                                ),
                            pitch_range=pitchtools.PitchRange(
                                range_string='[C4, D7]',
                                ),
                            sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                            ),
                        instrumenttools.Piccolo(
                            instrument_name='piccolo',
                            short_instrument_name='picc.',
                            instrument_name_markup=markuptools.Markup(
                                contents=['Piccolo'],
                                ),
                            short_instrument_name_markup=markuptools.Markup(
                                contents=['Picc.'],
                                ),
                            allowable_clefs=instrumenttools.ClefList(
                                [
                                    indicatortools.Clef(
                                        name='treble',
                                        ),
                                    ]
                                ),
                            pitch_range=pitchtools.PitchRange(
                                range_string='[D5, C8]',
                                ),
                            sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c''"),
                            ),
                        ]
                    ),
                )

    Performer models instrument doublings.
    '''

    ### INITIALIZER ###

    def __init__(self, name=None, instruments=None):
        from abjad.tools import instrumenttools
        self._instruments = instrumenttools.InstrumentList()
        self.name = name
        self.instruments = instruments

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a performer with name and instruments equal to
        those of this performer. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.name == argument.name:
                if self.instruments == argument.instruments:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats performer.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __hash__(self):
        r'''Hashes performer.

        Returns string.
        '''
        return hash((type(self).__name__, self.name, tuple(self.instruments)))

    ### PUBLIC METHODS ###

    def get_instrument(self, instrument_name):
        r'''Gets instrument in performer with `instrument_name`.

        ..  container:: example

            For examples:

            ::

                >>> flutist = instrumenttools.Performer(name='flutist')
                >>> flutist.instruments.append(instrumenttools.Flute())
                >>> flutist.instruments.append(instrumenttools.Piccolo())

        ..  container:: example

            **Example 1.** Gets instrument with instrument name:

            ::

                >>> flutist.get_instrument('piccolo')
                Piccolo()

        ..  container:: example

            **Example 2.** Gets instrument with short instrument name:

            ::

                >>> flutist.get_instrument('picc.')
                Piccolo()

        ..  container:: example

            **Example 3.** Gets instrument regardless of case:

            ::

                >>> flutist.get_instrument('PICCOLO')
                Piccolo()

        ..  container:: example

            **Example 4.** Returns none when no match is found:

            ::

                >>> flutist.get_instrument('xyl.') is None
                True

        Returns instrument or none.
        '''
        for instrument in self.instruments:
            if instrument.instrument_name.lower() == instrument_name.lower():
                return instrument
            if instrument.short_instrument_name.lower() == \
                instrument_name.lower():
                return instrument

    # TODO: make private
    @staticmethod
    def list_performer_names():
        r'''Lists performer names.

        ..  container:: example

            ::

                >>> for name in instrumenttools.Performer.list_performer_names():
                ...     name
                ...
                'accordionist'
                'alto'
                'baritone'
                'bass'
                'bassist'
                'bassoonist'
                'cellist'
                'clarinetist'
                'flutist'
                'guitarist'
                'harpist'
                'harpsichordist'
                'hornist'
                'mezzo-soprano'
                'oboist'
                'percussionist'
                'pianist'
                'saxophonist'
                'soprano'
                'tenor'
                'trombonist'
                'trumpeter'
                'tubist'
                'vibraphonist'
                'violinist'
                'violist'
                'xylophonist'

        Returns list.
        '''
        from abjad.tools import instrumenttools
        performer_names = set([])
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            performer_name = instrument._get_default_performer_name()
            performer_names.add(performer_name)
        return list(sorted(performer_names))

    # TODO: make private
    @staticmethod
    def list_primary_performer_names():
        r'''Lists primary performer names.

        ..  container:: example

            ::

                >>> for pair in instrumenttools.Performer.list_primary_performer_names():
                ...     pair
                ...
                ('accordionist', 'acc.')
                ('alto', 'alto')
                ('baritone', 'bar.')
                ('bass', 'bass')
                ('bassist', 'cb.')
                ('bassoonist', 'bsn.')
                ('cellist', 'vc.')
                ('clarinetist', 'cl.')
                ('flutist', 'fl.')
                ('guitarist', 'gt.')
                ('harpist', 'hp.')
                ('harpsichordist', 'hpschd.')
                ('hornist', 'hn.')
                ('mezzo-soprano', 'ms.')
                ('oboist', 'ob.')
                ('pianist', 'pf.')
                ('saxophonist', 'alt. sax.')
                ('soprano', 'sop.')
                ('tenor', 'ten.')
                ('trombonist', 'ten. trb.')
                ('trumpeter', 'tp.')
                ('tubist', 'tb.')
                ('violinist', 'vn.')
                ('violist', 'va.')

        Returns list.
        '''
        from abjad.tools import instrumenttools
        performer_names = set([])
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            if instrument._is_primary_instrument:
                performer_name = instrument._get_default_performer_name()
                performer_abbreviation = getattr(
                    instrument, 'performer_abbreviation', None)
                performer_abbreviation = performer_abbreviation or \
                    instrument.short_instrument_name
                performer_names.add((performer_name, performer_abbreviation))
        return list(sorted(performer_names))

    # TODO: make private
    @staticmethod
    def make_performer_name_instrument_dictionary():
        r'''Makes performer name / instrument dictionary.

        ..  container:: example

            ::

                >>> dictionary = \
                ...     performer.make_performer_name_instrument_dictionary()
                >>> for key, value in sorted(dictionary.items()):
                ...     print(key + ':')
                ...     for x in value:
                ...         print('\t{}'.format(x.__name__))
                accordionist:
                    Accordion
                alto:
                    AltoVoice
                baritone:
                    BaritoneVoice
                bass:
                    BassVoice
                bassist:
                    Contrabass
                bassoonist:
                    Bassoon
                    Contrabassoon
                brass player:
                    AltoTrombone
                    BassTrombone
                    FrenchHorn
                    TenorTrombone
                    Trumpet
                    Tuba
                cellist:
                    Cello
                clarinetist:
                    BassClarinet
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    ContrabassClarinet
                clarinettist:
                    BassClarinet
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    ContrabassClarinet
                contrabassist:
                    Contrabass
                double reed player:
                    Bassoon
                    Contrabassoon
                    EnglishHorn
                    Oboe
                flautist:
                    AltoFlute
                    BassFlute
                    ContrabassFlute
                    Flute
                    Piccolo
                flutist:
                    AltoFlute
                    BassFlute
                    ContrabassFlute
                    Flute
                    Piccolo
                guitarist:
                    Guitar
                harpist:
                    Harp
                harpsichordist:
                    Harpsichord
                hornist:
                    FrenchHorn
                instrumentalist:
                    Accordion
                    AltoFlute
                    AltoSaxophone
                    AltoTrombone
                    AltoVoice
                    BaritoneSaxophone
                    BaritoneVoice
                    BassClarinet
                    BassFlute
                    Bassoon
                    BassSaxophone
                    BassTrombone
                    BassVoice
                    Cello
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    Contrabass
                    ContrabassClarinet
                    ContrabassFlute
                    Contrabassoon
                    ContrabassSaxophone
                    EnglishHorn
                    Flute
                    FrenchHorn
                    Glockenspiel
                    Guitar
                    Harp
                    Harpsichord
                    Marimba
                    MezzoSopranoVoice
                    Oboe
                    Percussion
                    Piano
                    Piccolo
                    SopraninoSaxophone
                    SopranoSaxophone
                    SopranoVoice
                    TenorSaxophone
                    TenorTrombone
                    TenorVoice
                    Trumpet
                    Tuba
                    Vibraphone
                    Viola
                    Violin
                    Xylophone
                keyboardist:
                    Accordion
                    Harpsichord
                    Piano
                mezzo-soprano:
                    MezzoSopranoVoice
                oboist:
                    EnglishHorn
                    Oboe
                percussionist:
                    Glockenspiel
                    Marimba
                    Percussion
                    Vibraphone
                    Xylophone
                pianist:
                    Piano
                reed player:
                    AltoSaxophone
                    BaritoneSaxophone
                    BassClarinet
                    Bassoon
                    BassSaxophone
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    ContrabassClarinet
                    Contrabassoon
                    ContrabassSaxophone
                    EnglishHorn
                    Oboe
                    SopraninoSaxophone
                    SopranoSaxophone
                    TenorSaxophone
                saxophonist:
                    AltoSaxophone
                    BaritoneSaxophone
                    BassSaxophone
                    ContrabassSaxophone
                    SopraninoSaxophone
                    SopranoSaxophone
                    TenorSaxophone
                single reed player:
                    AltoSaxophone
                    BaritoneSaxophone
                    BassClarinet
                    BassSaxophone
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    ContrabassClarinet
                    ContrabassSaxophone
                    SopraninoSaxophone
                    SopranoSaxophone
                    TenorSaxophone
                soprano:
                    SopranoVoice
                string player:
                    Cello
                    Contrabass
                    Guitar
                    Harp
                    Viola
                    Violin
                tenor:
                    TenorVoice
                trombonist:
                    AltoTrombone
                    BassTrombone
                    TenorTrombone
                trumpeter:
                    Trumpet
                tubist:
                    Tuba
                vibraphonist:
                    Vibraphone
                violinist:
                    Violin
                violist:
                    Viola
                vocalist:
                    AltoVoice
                    BaritoneVoice
                    BassVoice
                    MezzoSopranoVoice
                    SopranoVoice
                    TenorVoice
                wind player:
                    AltoFlute
                    AltoSaxophone
                    BaritoneSaxophone
                    BassClarinet
                    BassFlute
                    Bassoon
                    BassSaxophone
                    ClarinetInA
                    ClarinetInBFlat
                    ClarinetInEFlat
                    ContrabassClarinet
                    ContrabassFlute
                    Contrabassoon
                    ContrabassSaxophone
                    EnglishHorn
                    Flute
                    FrenchHorn
                    Oboe
                    Piccolo
                    SopraninoSaxophone
                    SopranoSaxophone
                    TenorSaxophone
                xylophonist:
                    Xylophone

        Returns ordered dictionary.
        '''
        from abjad.tools import instrumenttools
        result = collections.OrderedDict()
        for instrument_class in instrumenttools.Instrument._list_instruments():
            instrument = instrument_class()
            for performer_name in instrument._get_performer_names():
                if performer_name in result:
                    result[performer_name].append(instrument_class)
                else:
                    result[performer_name] = [instrument_class]
        for instruments in result.values():
            instruments.sort(key=lambda x: x.__name__.lower())
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def instrument_count(self):
        r'''Gets count of instruments to be played by performer.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> performer.instrument_count
                2

        Returns nonnegative integer
        '''
        return len(self.instruments)

    @property
    def instruments(self):
        r'''Gets and sets instruments to be played by performer.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> for instrument in performer.instruments:
                ...     instrument
                ...
                Flute()
                Piccolo()

        Returns instrument list.
        '''
        return self._instruments

    @instruments.setter
    def instruments(self, instruments):
        from abjad.tools.instrumenttools.Instrument import Instrument
        if instruments is None:
            self._instruments[:] = []
        elif isinstance(instruments,
            (list, datastructuretools.TypedList)):
            assert all(isinstance(x, Instrument) for x in instruments)
            self._instruments[:] = instruments[:]
        else:
            message = 'instruments {!r} must be list or none.'
            message = message.format(instruments)
            raise TypeError(message)

    @property
    def is_doubling(self):
        r'''Is true when performer is to play more than one instrument.
        Otherwise false.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> performer.is_doubling
                True

        Returns true or false.
        '''
        return 1 < self.instrument_count

    # TODO: make private
    @property
    def likely_instruments_based_on_performer_name(self):
        r'''Gets likely instruments based on performer name.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> for likely_instrument in \
                ...     performer.likely_instruments_based_on_performer_name:
                ...     likely_instrument.__name__
                ...
                'AltoFlute'
                'BassFlute'
                'ContrabassFlute'
                'Flute'
                'Piccolo'

        Returns list.
        '''
        dictionary = self.make_performer_name_instrument_dictionary()
        try:
            result = dictionary[self.name]
        except KeyError:
            result = []
        return result

    # TODO: make private
    @property
    def most_likely_instrument_based_on_performer_name(self):
        r'''Gets most likely instrument based on performer name.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> performer.most_likely_instrument_based_on_performer_name
                <class 'abjad.tools.instrumenttools.Flute.Flute'>

        Returns instrument class.
        '''
        for likely_instrument_class in \
            self.likely_instruments_based_on_performer_name:
            likely_instrument = likely_instrument_class()
            if likely_instrument._is_primary_instrument:
                return likely_instrument_class

    @property
    def name(self):
        r'''Gets and sets score name of performer.

        ..  container:: example

            ::

                >>> performer = instrumenttools.Performer(name='flutist')
                >>> performer.instruments.append(instrumenttools.Flute())
                >>> performer.instruments.append(instrumenttools.Piccolo())

            ::

                >>> performer.name
                'flutist'

        Returns string.
        '''
        return self._name

    @name.setter
    def name(self, name):
        assert isinstance(name, (str, type(None)))
        self._name = name



# -*- coding: utf-8 -*-
import abjad


metadata = abjad.datastructuretools.TypedOrderedDict(
    [
        ('composer', {composer_last_name!r}),
        ('title', {score_title!r}),
        ('year', {year}),
        ]
    )



# -*- coding: utf-8 -*-


def to_tridirectional_lilypond_symbol(argument):
    r'''Changes `argument` to tridirectional LilyPond symbol.

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(Up)
            '^'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol('neutral')
            '-'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol('default')
            '-'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(Down)
            '_'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(1)
            '^'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(0)
            '-'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(-1)
            '_'

    Returns none when `argument` is none.

    Returns `argument` when `argument` is `'^'`, `'-'` or `'_'`.

    Returns string or none.
    '''
    lookup = {
        Up: '^',
        '^': '^',
        'up': '^',
        1: '^',
        Down: '_',
        '_': '_',
        'down': '_',
        -1: '_',
        Center: '-',
        '-': '-',
        0: '-',
        'center': '-',
        'default': '-',
        'neutral': '-',
        }
    if argument is None:
        return None
    elif argument in lookup:
        return lookup[argument]
    raise ValueError(argument)



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/build/.gitignore',
        'test_score/test_score/build/assets/.gitignore',
        'test_score/test_score/build/assets/instrumentation.tex',
        'test_score/test_score/build/assets/performance-notes.tex',
        'test_score/test_score/build/parts.ily',
        'test_score/test_score/build/segments.ily',
        'test_score/test_score/build/segments/.gitignore',
        'test_score/test_score/build/segments/segment-one.ily',
        'test_score/test_score/build/segments/segment-three.ily',
        'test_score/test_score/build/segments/segment-two.ily',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success(self, open_file_mock):
        self.create_score()
        self.create_segment('segment_one')
        self.create_segment('segment_two')
        self.create_segment('segment_three')
        self.illustrate_segments()
        collect_script = commandlinetools.ManageSegmentScript()
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    collect_script(['--collect'])
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Collecting segments:
            segments/segment_one/illustration.ly --> build/segments/segment-one.ily
            segments/segment_three/illustration.ly --> build/segments/segment-three.ily
            segments/segment_two/illustration.ly --> build/segments/segment-two.ily
            Reading test_score/segments/metadata.json ... OK!
        '''.replace('/', os.path.sep))
        self.compare_path_contents(self.build_path, self.expected_files)
        path = self.build_path.joinpath('segments.ily')
        self.compare_lilypond_contents(path, r'''
        {
            \include "../segments/segment-one.ily"
            \include "../segments/segment-two.ily"
            \include "../segments/segment-three.ily"
        }
        '''.replace('/', os.path.sep))
        path = self.build_path.joinpath('segments', 'segment-one.ily')
        self.compare_lilypond_contents(path, r'''
        \context Score = "Example Score" <<
            \context Staff = "Example Staff" {
                \context Voice = "Example Voice" {
                    c'4 (
                    d'4
                    e'4
                    f'4 )
                }
            }
        >>
        ''')
        path = self.build_path.joinpath('segments', 'segment-two.ily')
        self.compare_lilypond_contents(path, r'''
        \context Score = "Example Score" <<
            \context Staff = "Example Staff" {
                \context Voice = "Example Voice" {
                    c'4 (
                    d'4
                    e'4
                    f'4 )
                }
            }
        >>
        ''')
        path = self.build_path.joinpath('segments', 'segment-three.ily')
        self.compare_lilypond_contents(path, r'''
        \context Score = "Example Score" <<
            \context Staff = "Example Staff" {
                \context Voice = "Example Voice" {
                    c'4 (
                    d'4
                    e'4
                    f'4 )
                }
            }
        >>
        ''')



# -*- coding: utf-8 -*-
import copy
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class TreeNode(AbjadObject):
    r'''A node.

    Node in a generalized tree.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        '_parent',
        )

    ### INITIALIZER ###

    def __init__(self, name=None):
        self._name = None
        self._parent = None
        self.name = name

    ### SPECIAL METHODS ###

    def __copy__(self):
        r'''Copies tree node.
        '''
        import copy
        agent = systemtools.StorageFormatAgent(self)
        arguments = []
        for value in agent.get_template_dict().values():
            if isinstance(value, tuple):
                value = tuple(copy.copy(_) for _ in value)
            else:
                value = copy.copy(value)
            arguments.append(value)
        return type(self)(*arguments)

    ### PRIVATE METHODS ###

    def _cache_named_children(self):
        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.items():
                name_dictionary[name] = copy.copy(children)
        if hasattr(self, 'name') and self.name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = set([])
            name_dictionary[self.name].add(self)
        return name_dictionary

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = agent.signature_names
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=True,
            template_names=names,
            )

    def _get_node_state_flags(self):
        state_flags = {}
        for name in self._state_flag_names:
            state_flags[name] = True
            for node in self.improper_parentage:
                if not getattr(node, name):
                    state_flags[name] = False
                    break
        return state_flags

    def _mark_entire_tree_for_later_update(self):
        for node in self.improper_parentage:
            for name in self._state_flag_names:
                setattr(node, name, False)

    def _remove_from_parent(self):
        if self._parent is not None:
            index = self._parent.index(self)
            self._parent._children.pop(index)

    def _remove_named_children_from_parentage(self, name_dictionary):
        if self._parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    for node in name_dictionary[name]:
                        named_children[name].remove(node)
                    if not named_children[name]:
                        del named_children[name]

    def _restore_named_children_to_parentage(self, name_dictionary):
        if self._parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].update(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])

    def _set_parent(self, new_parent):
        named_children = self._cache_named_children()
        self._remove_named_children_from_parentage(named_children)
        self._remove_from_parent()
        self._parent = new_parent
        self._restore_named_children_to_parentage(named_children)
        self._mark_entire_tree_for_later_update()

    ### PRIVATE PROPERTIES ###

    @property
    def _state_flag_names(self):
        return ()

    ### PUBLIC PROPERTIES ###

    @property
    def depth(self):
        r'''The depth of a node in a rhythm-tree structure.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()
            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.depth
            0

        ::

            >>> a[0].depth
            1

        ::

            >>> a[0][0].depth
            2

        Returns int.
        '''
        node = self
        depth = 0
        while node.parent is not None:
            depth += 1
            node = node.parent
        return depth

    @property
    def depthwise_inventory(self):
        r'''A dictionary of all nodes in a rhythm-tree, organized by their
        depth relative the root node.

        ::

            >>> a = datastructuretools.TreeContainer(name='a')
            >>> b = datastructuretools.TreeContainer(name='b')
            >>> c = datastructuretools.TreeContainer(name='c')
            >>> d = datastructuretools.TreeContainer(name='d')
            >>> e = datastructuretools.TreeContainer(name='e')
            >>> f = datastructuretools.TreeContainer(name='f')
            >>> g = datastructuretools.TreeContainer(name='g')

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])
            >>> c.extend([f, g])

        ::

            >>> inventory = a.depthwise_inventory
            >>> for depth in sorted(inventory):
            ...     print('DEPTH: {}'.format(depth))
            ...     for node in inventory[depth]:
            ...         print(node.name)
            ...
            DEPTH: 0
            a
            DEPTH: 1
            b
            c
            DEPTH: 2
            d
            e
            f
            g

        Returns dictionary.
        '''
        def recurse(node):
            if node.depth not in inventory:
                inventory[node.depth] = []
            inventory[node.depth].append(node)
            if hasattr(node, 'children'):
                for child in node.children:
                    recurse(child)
        inventory = {}
        recurse(self)
        return inventory

    @property
    def graph_order(self):
        r'''Graph order of tree node.

        Returns tuple.
        '''
        from abjad.tools import sequencetools
        order = []
        components = sequencetools.Sequence(reversed(self.improper_parentage))
        for parent, child in components.nwise():
            order.append(parent.index(child))
        return tuple(order)

    @property
    def improper_parentage(self):
        r'''The improper parentage of a node in a rhythm-tree, being the
        sequence of node beginning with itself and ending with the root node
        of the tree.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.improper_parentage == (a,)
            True

        ::

            >>> b.improper_parentage == (b, a)
            True

        ::

            >>> c.improper_parentage == (c, b, a)
            True

        Returns tuple of tree nodes.
        '''
        node = self
        parentage = [node]
        while node.parent is not None:
            node = node.parent
            parentage.append(node)
        return tuple(parentage)

    @property
    def parent(self):
        r'''Parent of tree node.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.parent is None
            True

        ::

            >>> b.parent is a
            True

        ::

            >>> c.parent is b
            True

        Returns tree node.
        '''
        return self._parent

    @property
    def proper_parentage(self):
        r'''The proper parentage of a node in a rhythm-tree, being the
        sequence of node beginning with the node's immediate parent and
        ending with the root node of the tree.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.proper_parentage == ()
            True

        ::

            >>> b.proper_parentage == (a,)
            True

        ::

            >>> c.proper_parentage == (b, a)
            True

        Returns tuple of tree nodes.
        '''
        return self.improper_parentage[1:]

    @property
    def root(self):
        r'''The root node of the tree: that node in the tree which has
        no parent.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.root is a
            True
            >>> b.root is a
            True
            >>> c.root is a
            True

        Returns tree node.
        '''
        node = self
        while node.parent is not None:
            node = node.parent
        return node

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Named of tree node.

        Returns string.
        '''
        return self._name

    @name.setter
    def name(self, argument):
        assert isinstance(argument, (str, type(None)))
        old_name = self._name
        for parent in self.proper_parentage:
            named_children = parent._named_children
            if old_name is not None:
                named_children[old_name].remove(self)
                if not named_children[old_name]:
                    del named_children[old_name]
            if argument is not None:
                if argument not in named_children:
                    named_children[argument] = set([self])
                else:
                    named_children[argument].add(self)
        self._name = argument



# -*- coding: utf-8 -*-
from abjad.demos.ferneyhough.make_row_of_nested_tuplets \
    import make_row_of_nested_tuplets


def make_rows_of_nested_tuplets(tuplet_duration, row_count, column_count):
    r'''Makes rows of nested tuplets.
    '''

    assert 0 < row_count
    rows_of_nested_tuplets = []
    for n in range(row_count):
        outer_tuplet_proportions = (1, n + 1)
        row_of_nested_tuplets = make_row_of_nested_tuplets(
            tuplet_duration, outer_tuplet_proportions, column_count)
        rows_of_nested_tuplets.append(row_of_nested_tuplets)
    return rows_of_nested_tuplets



# -*- coding: utf-8 -*-


# TODO: make private and add to yield_all_integer_partitions()
def next_integer_partition(integer_partition):
    r'''Gets next integer partition following `integer_partition`
    in descending lex order.

    ..  container:: example

        ::

            >>> mathtools.next_integer_partition((8, 3))
            (8, 2, 1)

        ::

            >>> mathtools.next_integer_partition((8, 2, 1))
            (8, 1, 1, 1)

        ::

            >>> mathtools.next_integer_partition((8, 1, 1, 1))
            (7, 4)

    `integer_partition` must be tuple of positive integers.

    Returns new integer partition tuple.
    '''
    _validate_input(integer_partition)
    left_half, right_half = _split_into_left_and_right_halves(
        integer_partition
        )
    # if input was all 1s like (1, 1, 1, 1) then we're done
    if not left_half:
        return None
    new_left_half = left_half[:-1] + [left_half[-1] - 1]
    new_right_weight = sum(right_half) + 1
    new_right_half = _rewrite(new_right_weight, new_left_half[-1])
    result = new_left_half + new_right_half
    result = tuple(result)
    return result


def _split_into_left_and_right_halves(integer_partition):
    r'''Splits not-1s (left half) from 1s (right half).

    _split_into_left_and_right_halves((8, 3))
    [8, 3], []

    _split_into_left_and_right_halves((8, 2, 1))
    [8, 2], [1, ]

    _split_into_left_and_right_halves((8, 1, 1, 1))
    [8], [1, 1, 1]

    '''
    left_half = []
    right_half = []
    for part in integer_partition:
        if not part == 1:
            left_half.append(part)
        else:
            right_half.append(part)
    return left_half, right_half


def _validate_input(integer_partition):
    r'''Ensures monotonically decreasing iterable of positive integers.

    (8, 2, 2, 1) is OK.
    (8, 1, 2, 2) is not.

    '''
    previous = None
    for current in integer_partition:
        if not isinstance(current, int):
            message = 'must be integer.'
            raise TypeError(message)
        if not 0 < current:
            message = 'must be positive.'
            raise ValueError(message)
        if previous is not None:
            if not current <= previous:
                message = 'parts must decrease monotonically.'
                raise ValueError(message)


def _rewrite(n, m):
    r'''Write positive integer n as the sum of many m in a row, followed either
    by nothing or by a final positive integer p such that p is strictly less
    than m.

        _rewrite(8, 4)
        (4, 4)

        _rewrite(8, 3)
        (3, 3, 2)

        _rewrite(8, 2)
        (2, 2, 2, 2)

        _rewrite(8, 1)
        (1, 1, 1, 1, 1, 1, 1, 1)

    '''
    quotient = int(n / m)
    remainder = n % m
    if remainder:
        return [m] * quotient + [remainder]
    else:
        return [m] * quotient



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer___contains___01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()

    subcontainer = datastructuretools.TreeContainer([leaf_b])

    container = datastructuretools.TreeContainer([leaf_a, subcontainer])

    assert leaf_a in container
    assert leaf_b not in container
    assert leaf_c not in container



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_measure_number_01():
    r'''Measures in staff number correctly starting from 1.
    '''

    time_signature_pairs = [(3, 16), (5, 16), (5, 16)]
    measures = scoretools.make_spacer_skip_measures(time_signature_pairs)
    staff = Staff(measures)

    assert staff[0].measure_number == 1
    assert staff[1].measure_number == 2
    assert staff[2].measure_number == 3


def test_scoretools_Measure_measure_number_02():
    r'''Orphan measures number correctly starting from 1.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8")
    assert measure.measure_number == 1


def test_scoretools_Measure_measure_number_03():
    r'''Measure numbering works correctly after contents rotation.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert staff[0].measure_number == 1
    assert staff[1].measure_number == 2
    assert staff[2].measure_number == 3

    staff[:] = [staff[1], staff[2], staff[0]]

    assert staff[0].measure_number == 1
    assert staff[1].measure_number == 2
    assert staff[2].measure_number == 3

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                c'8
                d'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import selectiontools


def make_repeated_skips_from_time_signatures(time_signatures):
    r'''Make repeated skips from `time_signatures`:

    ::

        scoretools.make_repeated_skips_from_time_signatures([(2, 8), (3, 32)])
        [Selection(Skip('s8'), Skip('s8')), Selection(Skip('s32'), Skip('s32'), Skip('s32'))]

    Returns two-dimensional list of newly constructed skip lists.
    '''
    from abjad.tools import scoretools

    # init result
    result = []

    # iterate time signatures and make skips
    for time_signature in time_signatures:
        skips = _make_repeated_skips_from_time_signature(time_signature)
        result.append(skips)

    # return result
    return result


def _make_repeated_skips_from_time_signature(time_signature):
    from abjad.tools import scoretools

    # afford basic input polymorphism
    time_signature = indicatortools.TimeSignature(time_signature)

    # check input
    if time_signature.has_non_power_of_two_denominator:
        message = 'TODO: extend this function for time signatures'
        message += ' with non-power-of-two denominators.'
        raise NotImplementedError(message)

    # make and return repeated skips
    skip = scoretools.Skip((1, time_signature.denominator))
    skips = time_signature.numerator * skip
    result = selectiontools.Selection(skips)



# -*- coding: utf-8 -*-
from abjad.tools.lilypondparsertools.ReducedLyParser import ReducedLyParser


def parse_reduced_ly_syntax(string):
    r'''Parse the reduced LilyPond rhythmic syntax:

        >>> string = '4 -4. 8.. 5/3 { } 4'
        >>> result = lilypondparsertools.parse_reduced_ly_syntax(string)

    ::

        >>> for x in result:
        ...     x
        ...
        Note("c'4")
        Rest('r4.')
        Note("c'8..")
        Tuplet(Multiplier(5, 3), '')
        Note("c'4")

    Returns list.
    '''

    return ReducedLyParser()(string)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Articulation___repr___01():
    r'''Repr of unattached articulation is evaluable.
    '''

    articulation_1 = Articulation('staccato')
    articulation_2 = eval(repr(articulation_1))

    assert isinstance(articulation_1, Articulation)
    assert isinstance(articulation_2, Articulation)
    assert articulation_1 == articulation_2



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree___eq___01():

    scale_degree = tonalanalysistools.ScaleDegree(2)
    u = tonalanalysistools.ScaleDegree(2)
    voice = tonalanalysistools.ScaleDegree(3)

    assert      scale_degree == scale_degree
    assert      scale_degree == u
    assert not scale_degree == voice

    assert      u == scale_degree
    assert      u == u
    assert not u == voice

    assert not voice == scale_degree
    assert not voice == u
    assert      voice == voice



# -*- coding: utf-8 -*-
from abjad import *


def test_schemetools_make_spacing_vector_01():

    spacing_vector = schemetools.make_spacing_vector(0, 0, 12, 0)
    assert spacing_vector == schemetools.SchemeVector(
        schemetools.SchemePair('basic-distance', 0),
        schemetools.SchemePair('minimum-distance', 0),
        schemetools.SchemePair('padding', 12),
        schemetools.SchemePair('stretchability', 0))



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Dynamic(AbjadValueObject):
    r'''Dynamic.

    ..  container:: example

        **Example 1.** Initializes from dynamic name:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> dynamic = Dynamic('f')
            >>> attach(dynamic, staff[0])

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8 \f
                d'8
                e'8
                f'8
            }

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Initializes from other dynamic:

        ::

            >>> dynamic_1 = Dynamic('f')
            >>> dynamic_2 = Dynamic(dynamic_1)

        ::

            >>> dynamic_1
            Dynamic(name='f')

        ::

            >>> dynamic_2
            Dynamic(name='f')

    ..  container:: example

        **Example 3.** Niente is possible, but provides no formatting.

        ::

            >>> dynamic = Dynamic('niente')
            >>> format(dynamic, 'lilypond')
            ''

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_name',
        )

    _format_slot = 'right'

    _composite_dynamic_name_to_steady_state_dynamic_name = {
        'fp': 'p',
        'sf': 'f',
        'sff': 'ff',
        'sfp': 'p',
        'sfpp': 'pp',
        'sffp': 'p',
        'sffpp': 'pp',
        'sfz': 'f',
        'sp': 'p',
        'spp': 'pp',
        'rfz': 'f',
        }

    _dynamic_name_to_dynamic_ordinal = {
        'ppppp': -6,
        'pppp': -5,
        'ppp': -4,
        'pp': -3,
        'p': -2,
        'niente': mathtools.NegativeInfinity(),
        'mp': -1,
        'mf': 1,
        'f': 2,
        'ff': 3,
        'fff': 4,
        'ffff': 5,
        'fffff': 6,
        }

    _dynamic_names = (
        'ppppp',
        'pppp',
        'ppp',
        'pp',
        'p',
        'mp',
        'mf',
        'f',
        'ff',
        'fff',
        'ffff',
        'fffff',
        'fp',
        'sf',
        'sff',
        'sp',
        'spp',
        'sfz',
        'sffz',
        'sfffz',
        'sffp',
        'sffpp',
        'sfp',
        'sfpp',
        'rfz',
        'niente',
        )

    _dynamic_ordinal_to_dynamic_name = {
        -6: 'ppppp',
        -5: 'pppp',
        -4: 'ppp',
        -3: 'pp',
        -2: 'p',
        -1: 'mp',
        mathtools.NegativeInfinity(): 'niente',
        1: 'mf',
        2: 'f',
        3: 'ff',
        4: 'fff',
        5: 'ffff',
        6: 'fffff',
        }

    _lilypond_dynamic_commands = [
        _ for _ in _dynamic_names if not _ == 'niente'
        ]

    ### INITIALIZER ###

    def __init__(self, name='f'):
        from abjad.tools import scoretools
        if isinstance(name, type(self)):
            name = name.name
        assert name in self._dynamic_names, repr(name)
        self._name = name
        self._default_scope = scoretools.Staff

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats dynamic.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            **Example 1.** Gets storage format of forte:

            ::

                >>> dynamic = Dynamic('f')
                >>> print(format(dynamic))
                indicatortools.Dynamic(
                    name='f',
                    )

        ..  container:: example

            **Example 2.** Gets LilyPond format of forte:

            ::

                >>> dynamic = Dynamic('f')
                >>> print(format(dynamic, 'lilypond'))
                \f

        Returns string.
        '''
        if format_specification == 'lilypond':
            if self.name == 'niente':
                return ''
            elif self.name not in self._lilypond_dynamic_commands:
                message = 'dynamic name {!r} is not a LilyPond dynamic command.'
                message = message.format(self.name)
                raise Exception(message)
            return self._get_lilypond_format()
        superclass = super(Dynamic, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self._name)

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, component_expression):
        from abjad.tools import scoretools
        if not isinstance(component_expression, scoretools.Leaf):
            return False
        if self.name not in self._lilypond_dynamic_commands:
            return False
        return True

    def _get_lilypond_format(self):
        return r'\{}'.format(self.name)

    ### PUBLIC METHODS ###

    @staticmethod
    def composite_dynamic_name_to_steady_state_dynamic_name(name):
        r'''Changes composite `name` to steady state dynamic name.

        ..  container:: example

            **Example 1.** Steady state of sfp is piano:

            ::

                >>> Dynamic.composite_dynamic_name_to_steady_state_dynamic_name('sfp')
                'p'

        ..  container:: example

            **Example 2.** Steady state of rfz is forte:

            ::

                >>> Dynamic.composite_dynamic_name_to_steady_state_dynamic_name('rfz')
                'f'

        Returns string.
        '''
        return Dynamic._composite_dynamic_name_to_steady_state_dynamic_name[
            name]

    @staticmethod
    def dynamic_name_to_dynamic_ordinal(name):
        r'''Changes `name` to dynamic ordinal.

        ..  container:: example

            **Example 1.** Louder dynamics change to positive integers:

            ::

                >>> Dynamic.dynamic_name_to_dynamic_ordinal('fff')
                4

        ..  container:: example

            **Example 2.** Niente changes to negative infinity:

            ::

                >>> Dynamic.dynamic_name_to_dynamic_ordinal('niente')
                NegativeInfinity

        Returns integer or negative infinity.
        '''
        try:
            return Dynamic._dynamic_name_to_dynamic_ordinal[name]
        except KeyError:
            name = Dynamic.composite_dynamic_name_to_steady_state_dynamic_name(
                name)
            return Dynamic._dynamic_name_to_dynamic_ordinal[name]

    @staticmethod
    def dynamic_ordinal_to_dynamic_name(dynamic_ordinal):
        r'''Changes `dynamic_ordinal` to dynamic name.

        ..  container:: example

            **Example 1.** Negative values change to quiet dynamics:

            ::

                >>> Dynamic.dynamic_ordinal_to_dynamic_name(-5)
                'pppp'

        ..  container:: example

            **Example 2.** Negative infinity changes to niente:

            ::

                >>> negative_infinity = mathtools.NegativeInfinity()
                >>> Dynamic.dynamic_ordinal_to_dynamic_name(negative_infinity)
                'niente'

        Returns string.
        '''
        if dynamic_ordinal == mathtools.NegativeInfinity():
            return 'niente'
        else:
            return Dynamic._dynamic_ordinal_to_dynamic_name[dynamic_ordinal]

    @staticmethod
    def is_dynamic_name(argument):
        r'''Is true when `argument` is dynamic name. Otherwise false.

        ..  container:: example

            **Example 1.** Some usual dynamic names:

            ::

                >>> Dynamic.is_dynamic_name('f')
                True

            ::

                >>> Dynamic.is_dynamic_name('sfz')
                True

        ..  container:: example

            **Example 2.** Niente is also a dynamic name:

            ::

                >>> Dynamic.is_dynamic_name('niente')
                True

        Returns true or false.
        '''
        return argument in Dynamic._dynamic_names

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of dynamic.

        ..  container:: example

            **Example 1.** Forte:

            ::

                >>> dynamic = Dynamic('f')
                >>> dynamic.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        ..  container:: example

            **Example 2.** Piano:

            ::

                >>> dynamic = Dynamic('p')
                >>> dynamic.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Dynamics are staff-scoped by default.

        Returns staff.
        '''
        return self._default_scope

    @property
    def name(self):
        r'''Gets name of dynamic.

        ..  container:: example

            **Example 1.** Forte:

            ::

                >>> Dynamic('f').name
                'f'

        ..  container:: example

            **Example 2.** Piano:

            ::

                >>> Dynamic('p').name
                'p'

        ..  container:: example

            **Example 3.** Double sforzando:

            ::

                >>> Dynamic('sffz').name
                'sffz'

        ..  container:: example

            **Example 4.** Double sforzando-piano:

            ::

                >>> Dynamic('sffp').name
                'sffp'

        Returns string.
        '''
        return self._name

    @property
    def ordinal(self):
        r'''Gets ordinal value of dynamic.

        ..  container:: example

            **Example 1.** Forte:

            ::

                >>> Dynamic('f').ordinal
                2

        ..  container:: example

            **Example 2.** Piano:

            ::

                >>> Dynamic('p').ordinal
                -2

        Returns integer.
        '''
        name = self.name
        if name in self._composite_dynamic_name_to_steady_state_dynamic_name:
            name = self._composite_dynamic_name_to_steady_state_dynamic_name[
                name]
        ordinal = self._dynamic_name_to_dynamic_ordinal[name]
        return ordinal



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension__initialize_by_start_and_stop_01():

    chord_suspension = tonalanalysistools.ChordSuspension(4, 3)

    assert chord_suspension.start == tonalanalysistools.ScaleDegree(4)
    assert chord_suspension.stop == tonalanalysistools.ScaleDegree(3)


def test_tonalanalysistools_ChordSuspension__initialize_by_start_and_stop_02():

    chord_suspension = tonalanalysistools.ChordSuspension(4, ('flat', 3))

    assert chord_suspension.start == tonalanalysistools.ScaleDegree(4)
    assert chord_suspension.stop == tonalanalysistools.ScaleDegree('flat', 3)



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Segment import Segment
from abjad.tools.topleveltools import new


class IntervalSegment(Segment):
    r'''Interval segment.

    ..  container:: example

        Initializes from string:

        ::

            >>> intervals = 'm2 M10 -aug4 P5'
            >>> pitchtools.IntervalSegment(intervals)
            IntervalSegment(['+m2', '+M10', '-aug4', '+P5'])

    ..  container:: example

        Initializes from pitch segment:

        ::

            >>> pitch_segment = pitchtools.PitchSegment("c d e f g a b c'")
            >>> pitchtools.IntervalSegment(pitch_segment)
            IntervalSegment(['+M2', '+M2', '+m2', '+M2', '+M2', '+M2', '+m2'])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        ):
        from abjad.tools import pitchtools
        if isinstance(items, pitchtools.PitchSegment):
            intervals = []
            for one, two in sequencetools.Sequence(items).nwise():
                intervals.append(one - two)
            items = intervals
        Segment.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedInterval

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedInterval

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Interval

    ### PUBLIC PROPERTIES ###

    @property
    def slope(self):
        r'''Slope of interval segment.

        The slope of a interval segment is the sum of its
        intervals divided by its length:

        ::

            >>> pitchtools.IntervalSegment([1, 2]).slope
            Multiplier(3, 2)

        Returns multiplier.
        '''
        return durationtools.Multiplier.from_float(
            sum([x.number for x in self])) / len(self)

    @property
    def spread(self):
        r'''Spread of interval segment.

        The maximum interval spanned by any combination of
        the intervals within a numbered interval segment.

        ::

            >>> pitchtools.IntervalSegment([1, 2, -3, 1, -2, 1]).spread
            NumberedInterval(4)

        ::

            >>> pitchtools.IntervalSegment([1, 1, 1, 2, -3, -2]).spread
            NumberedInterval(5)

        Returns numbered interval.
        '''
        from abjad.tools import pitchtools
        current = maximum = minimum = 0
        for x in self:
            current += float(x)
            if maximum < current:
                maximum = current
            if current < minimum:
                minimum = current
        return pitchtools.NumberedInterval(maximum - minimum)

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes interval segment from component `selection`.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
            >>> pitchtools.IntervalSegment.from_selection(
            ...     staff, item_class=pitchtools.NumberedInterval)
            IntervalSegment([2, 2, 1, 2, 2, 2, 1])

        Returns interval segment.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        pitches = [_ for _ in pitch_segment]
        intervals = (-x for x in mathtools.difference_series(pitches))
        return class_(
            items=intervals,
            item_class=item_class,
            )

    def has_duplicates(self):
        r'''True if segment has duplicate items. Otherwise false.

        ::

            >>> intervals = 'm2 M3 -aug4 m2 P5'
            >>> segment = pitchtools.IntervalSegment(intervals)
            >>> segment.has_duplicates()
            True

        ::

            >>> intervals = 'M3 -aug4 m2 P5'
            >>> segment = pitchtools.IntervalSegment(intervals)
            >>> segment.has_duplicates()
            False

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        return len(pitchtools.IntervalSet(self)) < len(self)

    def rotate(self, n=0):
        r'''Rotates interval segment by index `n`.

        Returns new interval segment.
        '''
        return new(self, self[-n:] + self[:-n])



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container__get_spanners_that_dominate_component_pair_01():
    r'''Returns Python list of (spanner, index) pairs.
    Each spanner dominates an empty slice between components.
    No spanners dominate voice[0:0].
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8
            }
            {
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                g'8 \glissando
                a'8 \stopTrillSpan
            }
        }
        '''
        )

    receipt = voice._get_spanners_that_dominate_component_pair(None, voice[0])

    assert len(receipt) == 0
    assert receipt == set([])


def test_scoretools_Container__get_spanners_that_dominate_component_pair_02():
    r'''Beam and trill both dominate crack at voice[1:1].
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8
            }
            {
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                g'8 \glissando
                a'8 \stopTrillSpan
            }
        }
        '''
        )

    pair = (voice[0], voice[1])
    receipt = voice._get_spanners_that_dominate_component_pair(*pair)

    assert len(receipt) == 2
    assert (beam, 2) in receipt
    assert (trill, 2) in receipt


def test_scoretools_Container__get_spanners_that_dominate_component_pair_03():
    r'''Glissando and trill both dominate crack at voice[2:2].
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8
            }
            {
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                g'8 \glissando
                a'8 \stopTrillSpan
            }
        }
        '''
        )

    pair = (voice[1], voice[2])
    receipt = voice._get_spanners_that_dominate_component_pair(*pair)

    assert len(receipt) == 2
    assert (glissando, 2) in receipt
    assert (trill, 4) in receipt


def test_scoretools_Container__get_spanners_that_dominate_component_pair_04():
    r'''No spanners dominate empty slice following voice.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[:4])
    glissando = spannertools.Glissando()
    attach(glissando, leaves[-4:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \startTrillSpan
                d'8
            }
            {
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                g'8 \glissando
                a'8 \stopTrillSpan
            }
        }
        '''
        )

    pair = (voice[2], None)
    receipt = voice._get_spanners_that_dominate_component_pair(*pair)

    assert len(receipt) == 0
    assert receipt == set([])
